# -*- encoding: utf-8 -*-

import requests
from datetime import datetime

from django.conf import settings
from django.contrib.gis.geos import Point
from django.utils.translation import ugettext as _
from geotrek.common.parsers import TourInSoftParser, RowImportError, GlobalImportError
from geotrek.tourism.models import TouristicContent, TouristicContentType, TouristicEvent, TouristicEventType


class NormandieMainParser(TourInSoftParser):
    delete_attachments = True
    model = TouristicContent
    eid = 'eid'
    constant_fields = {
        'published': True,
    }
    natural_keys = {
        'category': 'label',
        'type1': 'label',
    }
    non_fields = {
        'attachments': 'Photos',
    }
    field_options = {
        'name': {
            'required': True,
        },
        'geom': {
            'required': True,
        },
        'category': {
            'create': True,
        },
        'type1': {
            'create': True,
        },
    }

    def filter_attachments(self, src, val):
        if not val:
            return []
        return [subval.split('|') for subval in val.split('#') if subval.split('|')[0]]

    def filter_geom(self, src, val):
        lat, lng = val
        if lng == '' or lat == '':
            raise RowImportError(u"Required value for fields 'GmapLatitude' and 'GmapLongitude'.")
        geom = Point(float(lng), float(lat), srid=4326)  # WGS84
        geom.transform(settings.SRID)
        return geom

class Normandie61MainParser(NormandieMainParser):
    fields = {
        'eid': 'SyndicObjectID',
        'name': 'SyndicObjectName',
        'description_teaser': 'DescriptifSynthetique',
        'description': 'DescriptionCommerciale',
        'contact': ('Adresse1', 'Adresse1Suite', 'Adresse2', 'Adresse3', 'CP', 'Commune', 'Cedex', 'MoyenDeCom'),
        'email': 'MoyenDeCom',
        'website': 'MoyenDeCom',
        'geom': ('GmapLatitude', 'GmapLongitude'),
    }

    def filter_contact(self, src, val):
        (Adresse1, Adresse1Suite, Adresse2, Adresse3, CodePostal, Commune, Cedex, MoyenDeCom) = val
        Telephone = None
        Mobile = None
        Fax = None
        if not MoyenDeCom:
            MoyenDeCom = ""
        for subval in MoyenDeCom.split('#'):
            if(subval.split('|')[0] == u'Téléphone filaire'):
                Telephone = subval.split('|')[1]
        for subval in MoyenDeCom.split('#'):
            if(subval.split('|')[0] == u'Téléphone cellulaire'):
                Mobile = subval.split('|')[1]
        for subval in MoyenDeCom.split('#'):
            if(subval.split('|')[0] == u'Télécopieur /fax'):
                Fax = subval.split('|')[1]

        lines = [line for line in [
            ' '.join([part for part in [Adresse1, Adresse1Suite] if part]),
            Adresse2,
            Adresse3,
            ' '.join([part for part in [CodePostal, Commune, Cedex] if part]),
            ' '.join([part for part in ['Tel:', Telephone] if Telephone]),
            ' '.join([part for part in ['Mobile:', Mobile] if Mobile]),
            ' '.join([part for part in ['Fax:', Fax] if Fax]),
        ] if line]
        return '<br>'.join(lines)

    def filter_email(self, src, val):
        if not val:
            return None
        for subval in val.split('#'):
            if(subval.split('|')[0] == u'Mél'):
                return subval.split('|')[1]

    def filter_website(self, src, val):
        if not val:
            return None
        for subval in val.split('#'):
            if(subval.split('|')[0] == u'Site web (URL)'):
                return subval.split('|')[1]

class NormandieAccomodation61Parser(Normandie61MainParser):
    constant_fields = {
        'category': u"Hébergements",
        'published': True,
    }

    def filter_type1(self, src, val):
        type1 = None
        if val == u"Meublés, locations, gîtes":
            type1 =  u"Gîtes"
        if val == u"Hébergements groupe":
            type1 =  u"Hébergements collectifs"
        if val == u"Chambre d'hôtes":
            type1 =  u"Chambres d'hôtes"
        if val == u"Insolite":
            type1 =  u"Hébergements insolites"
        if val in [u"Terrain de camping", u"Aire Naturelle", u"Camping rural", u"Parc résidentiel de loisir", u"Camping à la ferme", u"Camping"]:
            type1 =  u"Camping"
        if val == u"Aire de camping-car":
            type1 =  u"Camping-car"
        if type1:
            type1_val, created = (TouristicContentType.objects.get_or_create(label=type1, in_list=1, category=self.obj.category))
            if created:
                self.add_warning(_(u"type1 '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type1_val))
            return [type1_val]
        raise RowImportError(u"No type1 {type1}".format(type1=type1))

class Normandie72MainParser(NormandieMainParser):
    base_url = 'http://cdt72.media.tourinsoft.eu/upload/'
    fields = {
        'eid': 'SyndicObjectID',
        'name': 'SyndicObjectName',
        'description_teaser': 'DescriptifSynthetique',
        'description': 'Descriptif',
        'contact': ('Adresse1', 'Adresse1Suite', 'Adresse2', 'Adresse3', 'CodePostal', 'Commune', 'Cedex', 'CommTel', 'CommMob'),
        'email': 'CommMail',
        'website': 'CommWeb',
        'geom': ('GmapLatitude', 'GmapLongitude'),
    }

    @property
    def items(self):
        return self.root['value']

    def next_row(self):
        skip = 0
        while True:
            params = {
                '$format': 'json',
                '$inlinecount': 'allpages',
                '$top': 1000,
                '$skip': skip,
            }
            response = requests.get(self.url, params=params)
            if response.status_code != 200:
                raise GlobalImportError(_(u"Failed to download {url}. HTTP status code {status_code}").format(url=self.url, status_code=response.status_code))
            self.root = response.json()
            self.nb = int(self.root['odata.count'])
            for row in self.items:
                yield {self.normalize_field_name(src): val for src, val in row.iteritems()}
            skip += 1000
            if skip >= self.nb:
                return

    def filter_attachments(self, src, val):
        if not val:
            return []
        return [subval.split('||') for subval in val.split('##') if subval.split('||')[0]]

    def filter_contact(self, src, val):
        (Adresse1, Adresse1Suite, Adresse2, Adresse3, CodePostal, Commune, Cedex, Telephone, Mobile) = val
        lines = [line for line in [
            ' '.join([part for part in [Adresse1, Adresse1Suite] if part]),
            Adresse2,
            Adresse3,
            ' '.join([part for part in [CodePostal, Commune, Cedex] if part]),
            ' '.join([part for part in ['Tel:', Telephone] if Telephone]),
            ' '.join([part for part in ['Mobile:', Mobile] if Mobile]),
        ] if line]
        return '<br>'.join(lines)

class Restaurants_61(Normandie61MainParser):
    label = u"Restaurants - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    constant_fields = {
        'category': u"Restaurants",
        'published': True,
    }

class Restaurants_72(Normandie72MainParser):
    label = u"Restaurants - 72"
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    constant_fields = {
        'category': u"Restaurants",
        'published': True,
    }

class Camping_61(NormandieAccomodation61Parser):
    label = u"Hébergements - Camping - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_fields = {
        'type1': ('Classification'),
    }

class Hebergement_Location_61(NormandieAccomodation61Parser):
    label = u"Hébergements - Location - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_fields = {
        'type1': ('Classification'),
    }

    def __init__(self, *args, **kwargs):
        super(NormandieAccomodation61Parser, self).__init__(*args, **kwargs)
        self.fields['contact'] = ('Adresse1', 'Adresse1Suite', 'Adresse2', 'Adresse3', 'CP', 'Commune', 'Cedex', 'MoyenDeComReservation')
        self.fields['email'] = 'MoyenDeCom'
        self.fields['website'] = 'MoyenDeCom'

class Hotel_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Hébergements",
        'published': True,
    }
    label = u"Hébergements - Hôtel - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Hôtels"],
    }

class Location_Villages_vacances_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Hébergements",
        'published': True,
    }
    label = u"Hébergements - Gîte - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Gîtes"],
    }

class Hebergement_72(Normandie72MainParser):
    constant_fields = {
        'category': u"Hébergements",
        'published': True,
    }
    label = u"Hébergements - 72"
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_fields = {
        'type1': ('ObjectTypeName', 'Type'),
    }

    def filter_type1(self, src, val):
        object_type, object_subtype = val
        type1 = None
        if object_type == u"Hôtellerie de plein air (camping)":
            type1 = u"Camping"
        if object_type == u"Aire de Camping Car":
            type1 = u"Camping-car"
        if object_type == u"Hébergements locatifs (meublés et chambres d'hôtes)":
            if object_subtype in [u"Meublés", u"Locatif sur camping"]:
                type1 = u"Gîtes"
            if object_subtype == u"Chambres d'hôtes":
                type1 = u"Chambres d'hôtes"
            if object_subtype == u"Hébergement insolite":
                type1 = u"Hébergements insolites"
        if object_type == u"Hébergements Collectifs":
            type1 = u"Hébergements collectifs"
        if object_type == u"Hôtellerie":
            if object_subtype in [u"Hôtel", u"Hôtel - Restaurant"]:
                type1 = u"Hôtels"
        if type1:
            type1_val, created = (TouristicContentType.objects.get_or_create(label=type1, in_list=1, category=self.obj.category))
            if created:
                self.add_warning(_(u"type1 '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type1_val))
            return [type1_val]
        self.add_warning(_(u"No parsing defined for type1 '{val}' ").format(val=type1))
        return []

class Loisirs_sports_culture_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Sport et culture - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_fields = {
        'type1': ('ClassificationActivitesCulturelles', 'ClassificationActivitesSportives'),
    }
    def filter_type1(self, src, val):
        culture_val, sport_val = val
        type1 = None
        if culture_val is not None:
            type1 =  u"Activités culturelles"
        if sport_val is not None:
            type1 =  u"Activités sport et nature"
        if type1:
            type1_val, created = (TouristicContentType.objects.get_or_create(label=type1, in_list=1, category=self.obj.category))
            if created:
                self.add_warning(_(u"type1 '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type1_val))
            return [type1_val]
        raise RowImportError(u"No type1 {type1}".format(type1=type1))

class Loisirs_nature_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Patrimoine naturel - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Patrimoine naturel"],
    }

class Loisirs_degustation_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Dégustation - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Dégustation"],
    }

class Musees_61(Normandie61MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Musées et patrimoine culturel - 61"
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Musées - Patrimoine culturel"],
    }

class Loisirs_culture_72(Normandie72MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Activités culturelles - 72"
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Activités culturelles"],
    }

class Loisirs_sports_nature_72(Normandie72MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Activités sport et nature - 72"
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_constant_fields = {
        'type1': [u"Activités sport et nature"],
    }

class Loisirs_decouverte_72(Normandie72MainParser):
    constant_fields = {
        'category': u"Loisirs et découvertes",
        'published': True,
    }
    label = u"Loisirs et découvertes - Découverte - 72"
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    m2m_fields = {
        'type1': ('ObjectTypeName'),
    }
    def filter_type1(self, src, val):
        type1 = None
        if val == u"Patrimoine naturel":
            type1 =  u"Patrimoine naturel"
        if val == u"D\u00e9gustations (tous produits)":
            type1 =  u"Dégustation"
        if val == u"Patrimoine culturel":
            type1 =  u"Musées - Patrimoine culturel"
        if type1:
            type1_val, created = (TouristicContentType.objects.get_or_create(label=type1, in_list=1, category=self.obj.category))
            if created:
                self.add_warning(_(u"type1 '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type1_val))
            return [type1_val]
        raise RowImportError(u"No type1 {type1}".format(type1=type1))


class Fetes_Manifestations_61(Normandie61MainParser):
    model = TouristicEvent

    field_options = {
        'name': {
            'required': True,
        },
        'geom': {
            'required': True,
        },
        'type': {
            'create': True,
        },
    }

    fields = {
        'eid': 'SyndicObjectID',
        'name': 'SyndicObjectName',
        'description_teaser': 'DescriptifSynthetique',
        'description': 'DescriptionCommerciale',
        'contact': ('Adresse1', 'Adresse1Suite', 'Adresse2', 'Adresse3', 'CpLieuPrincipal', 'CommuneLieuPrincipal', 'Cedex', 'MoyenDeCom'),
        'email': 'MoyenDeCom',
        'website': 'MoyenDeCom',
        'geom': ('GmapLatitude', 'GmapLongitude'),
        'begin_date': 'PeriodeOuverture',
        'end_date': 'PeriodeOuverture',
        'type': ('CategorieManifestation', 'ThemeManifestation')
    }

    constant_fields = {
        'published': True,
    }

    label = u"Fêtes et Manifestations - 61"
        
    url = 'http://wcf.tourinsoft.com/Syndication/cdt61/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'

    def filter_begin_date(self, src, val):
        begin_date = None
        if val:
            begin_date_string = val.split('|')[0]
            if begin_date_string != '':
                begin_date = datetime.strptime(begin_date_string, "%d/%m/%Y")
        return begin_date

    def filter_end_date(self, src, val):
        end_date = None
        if val:
            end_date_string = val.split('|')[1]
            if end_date_string != '':
                end_date = datetime.strptime(end_date_string, "%d/%m/%Y")
        return end_date
    
    def filter_type(self, src, val):
        (categorie, theme) = val
        if not theme:
            theme = ()
        if not categorie:
            categorie = ()
        event_type = None
        if (u"Portes ouvertes" in categorie) or (u"Visite" in categorie) or (u"Manifestation Nationale" in categorie):
            event_type = u"Visites et sorties thématiques"
        if (u"Concert" in categorie) or (u"Danse" in categorie) or (u"Défilé Cortège Parade" in categorie) or (u"Opéra" in categorie) or (u"Spectacle" in categorie) or (u"Théâtre" in categorie) or (u"Festival" in categorie) or (u"Cinéma" in theme):
            event_type = u"Cinéma, Concerts, Festivals, Spectacles..."
        if (u"Foire ou salon" in categorie) or (u"Fête communale / patronale" in categorie) or (u"Son et lumières, Feux d'artifice" in categorie) or (u"Tradition et folklore" in categorie) or (u"Fêtes de fin d'année" in categorie) or (u"Illuminations" in categorie):
            event_type = u"Fêtes, Foires et salons"
        if (u"Compétition sportive" in categorie) or (u"Randonnées/Balades" in categorie):
            event_type = u"Randonnées et manifestations sportives"
        if (u"Vide greniers Braderie" in categorie) or (u"Marchés" in categorie):
            event_type = u"Marchés, brocantes et vide-greniers"
        if (u"Atelier / Stage" in categorie) or (u"Conférence" in categorie):
            event_type = u"Stages-Ateliers et conférences"
        if (u"Exposition" in categorie):
            event_type = u"Expositions"
        if (u"Contes et jeux" in categorie):
            event_type = u"Contes et lectures"
        if (u"Nature" in theme):
            event_type = u"Sorties nature / environnement"
        if (u"Nature" in categorie):
            event_type = u"Sorties nature / environnement"
        if event_type:
            type_val, created = (TouristicEventType.objects.get_or_create(type=event_type))
            if created:
                self.add_warning(_(u"type '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type_val))
            return type_val
        raise RowImportError(u"No type found")



class Fetes_Manifestations_72(Normandie72MainParser):
    model = TouristicEvent

    field_options = {
        'name': {
            'required': True,
        },
        'geom': {
            'required': True,
        },
        'type': {
            'create': True,
        },
    }

    fields = {
        'eid': 'SyndicObjectID',
        'name': 'SyndicObjectName',
        'description_teaser': 'DescriptifSynthetique',
        'description': 'Descriptif',
        'contact': ('Adresse1', 'Adresse1Suite', 'Adresse2', 'Adresse3', 'CodePostal', 'Commune', 'Cedex', 'CommTel', 'CommMob'),
        'email': 'CommMail',
        'website': 'CommWeb',
        'geom': ('GmapLatitude', 'GmapLongitude'),
        'begin_date': 'OuvertureGranule',
        'end_date': 'OuvertureGranule',
        'type': 'Categorie',
    }

    constant_fields = {
        'published': True,
    }

    label = u"Fêtes et Manifestations - 72"
        
    url = 'http://wcf.tourinsoft.com/Syndication/3.0/cdt72/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/Objects'
    
    def filter_begin_date(self, src, val):
        begin_date = None
        if val:
            begin_date_string = val.split('||')[0]
            if begin_date_string != '':
                begin_date = datetime.strptime(begin_date_string, "%d/%m/%Y")
        return begin_date

    def filter_end_date(self, src, val):
        end_date = None
        if val:
            end_date_string = val.split('||')[1]
            if end_date_string != '':
                end_date = datetime.strptime(end_date_string, "%d/%m/%Y")
        return end_date
    
    def filter_type(self, src, val):
        event_type = None
        if not val:
            val = ()
        if (u"Visites et sorties à thèmes" in val) or (u"Manifestation Nationale" in val) or (val is None):
            event_type = u"Visites et sorties thématiques"
        if (u"Concert" in val) or (u"Spectacle" in val) or (u"Théâtre" in val) or (u"Festival" in val) or (u"Cinéma" in val):
            event_type = u"Cinéma, Concerts, Festivals, Spectacles..."
        if (u"Foire ou salon" in val) or (u"Fêtes populaires" in val) or (u"Feux d'artifice" in val) or (u"Animations de Noël" in val):
            event_type = u"Fêtes, Foires et salons"
        if (u"Manifestation sportive" in val) or (u"Randonnée" in val) or (u"Rallye" in val):
            event_type = u"Randonnées et manifestations sportives"
        if (u"Brocantes & vide-greniers" in val) or (u"Marchés" in val):
            event_type = u"Marchés, brocantes et vide-greniers"
        if (u"Stages-Atelier-Jeux" in val) or (u"Conférence" in val):
            event_type = u"Stages-Ateliers et conférences"
        if (u"Exposition" in val):
            event_type = u"Expositions"
        if (u"Contes et lectures" in val):
            event_type = u"Contes et lectures"
        if (u"Sortie nature / environnement" in val):
            event_type = u"Sorties nature / environnement"
        if event_type:
            type_val, created = (TouristicEventType.objects.get_or_create(type=event_type))
            if created:
                self.add_warning(_(u"type '{val}' did not exist in Geotrek-Admin and was automatically created").format(val=type_val))
            return type_val
        raise RowImportError(u"No type {type}".format(type=event_type))
